/*
Copyright (C) 2006 by Intra2net AG - Gerd v. Egidy

The software in this package is distributed under the GNU General
Public License version 2 (with a special exception described below).

A copy of GNU General Public License (GPL) is included in this distribution,
in the file COPYING.GPL.

As a special exception, if other files instantiate templates or use macros
or inline functions from this file, or you compile this file and link it
with other works to produce a work based on this file, this file
does not by itself cause the resulting work to be covered
by the GNU General Public License.

However the source code for this file must still be made available
in accordance with section (3) of the GNU General Public License.

This exception does not invalidate any other reasons why a work based
on this file might be covered by the GNU General Public License.
*/
#ifndef __LIBT2N_COMMAND_CLIENT
#define __LIBT2N_COMMAND_CLIENT

#include <functional>
#include <string>

#include "client.hxx"
#include "container.hxx"

namespace libt2n
{

/// a client sending out commands to a server
class command_client
{
    public:
        static const int command_timeout_millisec_default=90000;
        static const int hello_timeout_millisec_default=30000;

    private:
        client_connection *c;

        int hello_timeout_millisec;
        int command_timeout_millisec;

        t2n_exception *constructorException;

        void read_hello();
        std::string read_packet(const int &millisec_timeout);
        bool check_hello(const std::string& hellostr);

    public:
        explicit command_client(client_connection* _c,
            int _command_timeout_millisec=command_timeout_millisec_default,
            int _hello_timeout_millisec=hello_timeout_millisec_default);

        virtual ~command_client();

        void replace_connection(client_connection* _c);

        void send_command(command* cmd, result_container &res);

        void set_command_timeout_millisec(int _command_timeout_millisec=command_timeout_millisec_default)
            { command_timeout_millisec=_command_timeout_millisec; }

        void set_hello_timeout_millisec(int _hello_timeout_millisec=hello_timeout_millisec_default)
            { hello_timeout_millisec=_hello_timeout_millisec; }

        int get_command_timeout_millisec() const
            { return command_timeout_millisec; }

        int get_hello_timeout_millisec() const
            { return hello_timeout_millisec; }

        bool is_connection_closed()
            { return c->is_closed(); }

        t2n_exception* get_constructor_exception()
            { return constructorException; }
};

}

#endif

