/*
Copyright (C) 2006 by Intra2net AG - Gerd v. Egidy

The software in this package is distributed under the GNU General
Public License version 2 (with a special exception described below).

A copy of GNU General Public License (GPL) is included in this distribution,
in the file COPYING.GPL.

As a special exception, if other files instantiate templates or use macros
or inline functions from this file, or you compile this file and link it
with other works to produce a work based on this file, this file
does not by itself cause the resulting work to be covered
by the GNU General Public License.

However the source code for this file must still be made available
in accordance with section (3) of the GNU General Public License.

This exception does not invalidate any other reasons why a work based
on this file might be covered by the GNU General Public License.
*/
#ifndef __LIBT2N_SOCKET_CLIENT
#define __LIBT2N_SOCKET_CLIENT

#include "client.hxx"
#include "socket_handler.hxx"

struct sockaddr;

namespace libt2n
{
/** @brief a connection from client to server using sockets.
    Use this class to connect from a client to a server.
 */
class socket_client_connection : public client_connection, public socket_handler
{
    public:
        static const int max_retries_default=5;
        static const int connect_timeout_millisec_default=30000;

    private:
        void real_write(const std::string& data)
            { socket_write(data); }

        void tcp_connect(int max_retries);
        void unix_connect(int max_retries);
        void connect_with_timeout(struct sockaddr *sock_addr,unsigned int sockaddr_size);

        int max_retries;
        int connect_timeout_millisec;

        std::string path;
        std::string server;
        int port;

        std::string lastErrorMsg;

    protected:

        std::ostream* get_logstream(log_level_values level)
            { return client_connection::get_logstream(level); }

    public:
        explicit socket_client_connection(int _port, const std::string& _server="127.0.0.1",
                                 int _connect_timeout_millisec=connect_timeout_millisec_default,
                                 int _max_retries=max_retries_default, std::ostream *_logstream=NULL,
                                 log_level_values _log_level=none);

        explicit socket_client_connection(const std::string& _path,
            int _connect_timeout_millisec=connect_timeout_millisec_default,
            int _max_retries=max_retries_default,
            std::ostream *_logstream=NULL, log_level_values _log_level=none);

        ~socket_client_connection();

        /** @brief read data from the socket and copy it into buffer
            @retval true if new data was found (does not mean that the received data 
                    is a complete packet though)
        */
        bool fill_buffer(int millisec_timeout)
            { return socket_handler::fill_buffer(buffer, millisec_timeout); }

        virtual void close();

        void reconnect();

        std::string get_last_error_msg()
            { return lastErrorMsg; }
};

}

#endif
